#include "system.h"
#include "smalltask.h"
#include "LED.h"


uint8_t Control_state = 0;
uint8_t note = 0;   //toon
uint8_t melody = 0;   //liedje
bool highpowermode = false;
bool darkmode = false;

void error_blink (void)
{
SET_ERR();
}


void Check_switch (void)
{
#define DARK_MODE 800                              // hold for 8 sec to turn off lights
#define HIGH_POWER 300                              // hold for 3 sec to switch between 500 and 1000mA
#define RESET_STATE 100                              // hold for 1 sec to go to default state
static uint16_t cntr = 0;
if (!GET_BUT())                                     // button pressed
    {
         cntr++;
    }
else                                                // button released
    {
        if (cntr > DARK_MODE)                      // pressed long enoug for dark mode
            {
                darkmode = !darkmode; 
                CLR_ERR();                                  // turn on error led
                Smalltask_rerun(TASK_error_blink, 500ms);   // turn off error led after 500ms
            }
        else if (cntr > HIGH_POWER)                      // pressed long enoug for high power mode
            {
                highpowermode = !highpowermode; 
                CLR_ERR();                                  // turn on error led
                Smalltask_rerun(TASK_error_blink, 500ms);   // turn off error led after 500ms
                TGL_MODE();                                 // toggle 500/1000mA setting of boost
            }
        else if (cntr > RESET_STATE)                  // pressed long enoug for reset mode
            {
                Control_state = 2;
            }  
        else if (cntr > 1)                            // short push
            {
                 Control_state++;
            }
        cntr = 0;
    }

Smalltask_rerun(TASK_Check_switch, 10ms);
#undef HIGH_POWER
#undef RESET_STATE
#undef DARK_MODE
}


void notes(void)
{
    switch(note)
    {
        case 0: 
            Smalltask_rerun(TASK_Control_task, 122);    //65.41Hz, note C2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(100);                        //0.7% on
                }
            __delay_us(100);                            //0.7% on
            CLR_PWREN();
            return;
        case 1: 
            Smalltask_rerun(TASK_Control_task, 115);    //69.30Hz, note C#2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(96);                        //0.7% on
                }
            __delay_us(96);                            //0.7% on
            CLR_PWREN();
            return;
        case 2: 
            Smalltask_rerun(TASK_Control_task, 109);    //73.42Hz, note D2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(90);                        //0.7% on
                }
            __delay_us(90);                            //0.7% on
            CLR_PWREN();
            return;
        case 3: 
            Smalltask_rerun(TASK_Control_task, 103);    //77.78Hz, note D#2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(85);                        //0.7% on
                }
            __delay_us(85);                            //0.7% on
            CLR_PWREN();
            return;  
        case 4:
            Smalltask_rerun(TASK_Control_task, 97);    //82.41Hz, note E2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(80);                        //0.7% on
                }
            __delay_us(80);                            //0.7% on
            CLR_PWREN();
            return;
        case 5: 
            Smalltask_rerun(TASK_Control_task, 92);    //87.31Hz, note F2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(75);                        //0.7% on
                }
            __delay_us(75);                            //0.7% on
            CLR_PWREN();
            return;
        case 6: 
            Smalltask_rerun(TASK_Control_task, 86);    //92.50Hz, note F#2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(71);                        //0.7% on
                }
            __delay_us(71);                            //0.7% on
            CLR_PWREN();
            return;
        case 7: 
            Smalltask_rerun(TASK_Control_task, 82);    //98.00Hz, note G2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(66);                        //0.7% on
                }
            __delay_us(66);                            //0.7% on
            CLR_PWREN();
            return; 
        case 8: 
            Smalltask_rerun(TASK_Control_task, 77);    //103.83Hz, note G#2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(62);                        //0.7% on
                }
            __delay_us(62);                            //0.7% on
            CLR_PWREN();
            return;
        case 9: 
            Smalltask_rerun(TASK_Control_task, 73);    //110.00Hz, note A2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(59);                        //0.7% on
                }
            __delay_us(59);                            //0.7% on
            CLR_PWREN();
            return;
        case 10: 
            Smalltask_rerun(TASK_Control_task, 67);    //116.54Hz, note A#2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(55);                        //0.7% on
                }
            __delay_us(55);                            //0.7% on
            CLR_PWREN();
            return;
        case 11: 
            Smalltask_rerun(TASK_Control_task, 65);    //123.47Hz, note B2
            SET_PWREN();
            if (highpowermode)                          //operating at 1A
                {
                __delay_us(51);                        //0.7% on
                }
            __delay_us(51);                            //0.7% on
            CLR_PWREN();
            return; 
        case 12: 
            Smalltask_rerun(TASK_Control_task, 7ms);
            return;
        default:
            Smalltask_rerun(TASK_Control_task, 6ms);
            return;
    }
    
}

void melody_task(void)      //this could be a melody by changing notes
{
    if (note > 13)
        note = 0;
    else
        note++;
    Smalltask_rerun(TASK_melody_task, 250ms);
}

void Control_task(void) //basic control states
{
    switch(Control_state)
    {
        case 0:
            Control_state++;
            CLR_ENA();                                      //enable boost converter and bypass precharge
            Smalltask_rerun(TASK_Control_task, 800ms);      //wait till the capacitors are full
            return;
        case 1:
            SET_ERR();                                      //error led off
            Control_state++;
            Check_switch();                                 //start polling the switch
            Led_task();                                     //start the fancy leds
            Smalltask_rerun(TASK_Control_task, 100ms);
            return;
        case 2:                                             //default operating state, teslacoil off
            Smalltask_rerun(TASK_Control_task, 100ms);
            return;
        case 3:
            Smalltask_rerun(TASK_Control_task, 200ms);  //5Hz
            SET_PWREN();
            __delay_us(1500);                            
            if (highpowermode)                          //operating at 1A
            {
                __delay_us(1500);                        
            }
            CLR_PWREN();
            return;
        case 4:
            Smalltask_rerun(TASK_Control_task, 100ms);  //10Hz
            SET_PWREN();
            __delay_us(750);                            //10Hz 750us 0.75%
            if (highpowermode)                          //operating at 1A
            {
                __delay_us(750);                        //10Hz 750us 0.75%
            }
            CLR_PWREN();            
            return;
        case 5:
            Smalltask_rerun(TASK_Control_task, 50ms);  //20Hz
            SET_PWREN();
            __delay_us(330);                            
            if (highpowermode)                          //operating at 1A
            {
                __delay_us(330);                        
            }
            CLR_PWREN();            
            return;
        case 6:
            note = 0;
            melody_task();
            Control_state++;
            Smalltask_rerun(TASK_Control_task, 100ms);
            return;
        case 7:
            notes();
            return;              
        case 8:
            Control_state = 2;
            Smalltask_rerun(TASK_Control_task, 100ms);
            return;
        default:
            Control_state = 2;
            Smalltask_rerun(TASK_Control_task, 100ms);
            return;
            
    }
}

